Index: include/vorbis/vorbisfile.h
===================================================================
RCS file: /usr/local/cvsroot/vorbis/include/vorbis/vorbisfile.h,v
retrieving revision 1.14
diff -u -r1.14 vorbisfile.h
--- include/vorbis/vorbisfile.h	2001/05/27 06:43:59	1.14
+++ include/vorbis/vorbisfile.h	2001/11/02 02:25:05
@@ -46,8 +46,9 @@
 #define  NOTOPEN   0
 #define  PARTOPEN  1
 #define  OPENED    2
-#define  STREAMSET 3
-#define  INITSET   4
+#define  STREAMFOUND    3
+#define  STREAMSET 4
+#define  INITSET   5
 
 typedef struct OggVorbis_File {
   void            *datasource; /* Pointer to a FILE *, etc. */
Index: lib/vorbisfile.c
===================================================================
RCS file: /usr/local/cvsroot/vorbis/lib/vorbisfile.c,v
retrieving revision 1.50
diff -u -r1.50 vorbisfile.c
--- lib/vorbisfile.c	2001/10/02 00:14:32	1.50
+++ lib/vorbisfile.c	2001/11/02 02:25:08
@@ -123,6 +123,13 @@
            advance the internal offset past the page end */
 	long ret=vf->offset;
 	vf->offset+=more;
+ 	if (vf->ready_state >= STREAMFOUND){
+ 	  int i;      
+          for(i=0; i<vf->links; i++)
+            if(vf->serialnos[i] == ogg_page_serialno(og)) break;
+          if(i==vf->links)  /* Just ignore unknown page .... */
+            ret = _get_next_page(vf,og,CHUNKSIZE);
+        }  
 	return(ret);
 	
       }
@@ -164,6 +171,7 @@
   return(offset);
 }
 
+	
 /* finds each bitstream link one at a time using a bisection search
    (has to begin by knowing the offset of the lb's initial page).
    Recurses for each link so it can alloc the link storage after
@@ -178,11 +186,47 @@
   long next=end;
   ogg_page og;
   long ret;
-  
+  ogg_int64_t *ignored;
+  int ignored_count=0;
+  long found=-1;
+
+{
+  int ignored_storage;
+  int passed_pages=0;
+
+  ignored = _ogg_malloc(4*sizeof(*ignored));
+  ignored_storage = 4;
+
+  _seek_helper(vf,begin);
+  while(1){
+    int ret = _get_next_page(vf, &og, CHUNKSIZE);
+    if(ret==OV_EREAD)return(OV_EREAD);
+    if(ogg_page_bos(&og)){
+      if(found != -1){
+        if(ignored_count == ignored_storage) {
+          ignored_storage += 4;
+          ignored = _ogg_realloc(ignored,ignored_storage * sizeof(*ignored));
+        }	  
+        ignored[ignored_count] = ogg_page_serialno(&og);
+        ignored_count++;
+	passed_pages=0;
+      } else {
+        /* XXX we should check this is really vorbis data here */
+        found = ret;
+	currentno = ogg_page_serialno(&og);
+      }
+    } else {
+      if((passed_pages >= 4) || ogg_page_eos(&og)) break;
+      passed_pages++;
+    }
+  }
+}  
+
   /* the below guards against garbage seperating the last and
      first pages of two links. */
   while(searched<endsearched){
     long bisect;
+    int i;
     
     if(endsearched-searched<CHUNKSIZE){
       bisect=searched;
@@ -193,7 +237,10 @@
     _seek_helper(vf,bisect);
     ret=_get_next_page(vf,&og,-1);
     if(ret==OV_EREAD)return(OV_EREAD);
-    if(ret<0 || ogg_page_serialno(&og)!=currentno){
+    for(i=0; i<ignored_count; i++)
+      if(ignored[i] == ogg_page_serialno(&og)) break;
+    if(ret<0 || ((currentno != ogg_page_serialno(&og)) && 
+		((i == ignored_count) || (ignored_count == 0)) ) ){  
       endsearched=bisect;
       if(ret>=0)next=ret;
     }else{
@@ -204,7 +251,7 @@
   _seek_helper(vf,next);
   ret=_get_next_page(vf,&og,-1);
   if(ret==OV_EREAD)return(OV_EREAD);
-  
+
   if(searched>=end || ret<0){
     vf->links=m+1;
     vf->offsets=_ogg_malloc((m+2)*sizeof(*vf->offsets));
@@ -215,7 +262,8 @@
     if(ret==OV_EREAD)return(OV_EREAD);
   }
   
-  vf->offsets[m]=begin;
+  vf->offsets[m]=found;
+  _ogg_free(ignored);
   return(0);
 }
 
@@ -225,7 +273,7 @@
 			  long *serialno,ogg_page *og_ptr){
   ogg_page og;
   ogg_packet op;
-  int i,ret=0;
+  int i, saved_state, ret=0;
   
   if(!og_ptr){
     ret=_get_next_page(vf,&og,CHUNKSIZE);
@@ -236,6 +284,12 @@
 
   if(serialno)*serialno=ogg_page_serialno(og_ptr);
   ogg_stream_init(&vf->os,ogg_page_serialno(og_ptr));
+  if(! vf->serialnos){ /* bootstrap */
+    vf->serialnos=_ogg_malloc(sizeof(*vf->serialnos));
+    vf->serialnos[0]=ogg_page_serialno(og_ptr);
+  }
+  
+  saved_state = vf->ready_state;
   vf->ready_state=STREAMSET;
   
   /* extract the initial header from the first page and verify that the
@@ -265,6 +319,7 @@
 	goto bail_header;
       }
   }
+  vf->ready_state = saved_state; 
   return 0; 
 
  bail_header:
@@ -293,7 +348,7 @@
   vf->vc=_ogg_realloc(vf->vc,vf->links*sizeof(*vf->vc));
   vf->dataoffsets=_ogg_malloc(vf->links*sizeof(*vf->dataoffsets));
   vf->pcmlengths=_ogg_malloc(vf->links*sizeof(*vf->pcmlengths));
-  vf->serialnos=_ogg_malloc(vf->links*sizeof(*vf->serialnos));
+  vf->serialnos=_ogg_realloc(vf->serialnos,vf->links*sizeof(*vf->pcmlengths));
   
   for(i=0;i<vf->links;i++){
     if(i==0){
@@ -304,7 +359,7 @@
       /* seek to the location of the initial header */
 
       _seek_helper(vf,vf->offsets[i]);
-      if(_fetch_headers(vf,vf->vi+i,vf->vc+i,NULL,NULL)<0){
+      if(_fetch_headers(vf,vf->vi+i,vf->vc+i,vf->serialnos+i ,NULL)<0){
     	vf->dataoffsets[i]=-1;
       }else{
 	vf->dataoffsets[i]=vf->offset;
@@ -312,8 +367,8 @@
       }
     }
 
-    /* get the serial number and PCM length of this link. To do this,
-       get the last page of the stream */
+    /* get the  PCM length of this link. To do this, get the last page 
+     * of the stream, and rewind to find our stream's tail.  .. */
     {
       long end=vf->offsets[i+1];
       _seek_helper(vf,end);
@@ -326,8 +381,8 @@
 	  vorbis_comment_clear(vf->vc+i);
 	  break;
 	}
-	if(ogg_page_granulepos(&og)!=-1){
-	  vf->serialnos[i]=ogg_page_serialno(&og);
+	if((ogg_page_granulepos(&og)!=-1) &&
+			(ogg_page_serialno(&og) == vf->serialnos[i])){
 	  vf->pcmlengths[i]=ogg_page_granulepos(&og);
 	  break;
 	}
@@ -390,6 +445,8 @@
 
   /* the initial header memory is referenced by vf after; don't free it */
   _prefetch_all_headers(vf,dataoffset);
+     if(vf->ready_state < STREAMFOUND)
+       vf->ready_state = STREAMFOUND;
   return(ov_raw_seek(vf,0));
 }
 
@@ -598,17 +655,20 @@
 }
 
 static int _ov_open2(OggVorbis_File *vf){
+  int ret;	
   if(vf->ready_state < OPENED)
     vf->ready_state=OPENED;
   if(vf->seekable){
-    int ret=_open_seekable2(vf);
+    ret=_open_seekable2(vf);
     if(ret){
       vf->datasource=NULL;
       ov_clear(vf);
     }
-    return(ret);
-  }
-  return 0;
+  } else
+    ret = 0;
+  if(vf->ready_state < STREAMFOUND)
+    vf->ready_state=STREAMFOUND;
+  return(ret);
 }
 
 
@@ -839,7 +899,7 @@
 int ov_raw_seek(OggVorbis_File *vf,long pos){
   ogg_stream_state work_os;
 
-  if(vf->ready_state<OPENED)return(OV_EINVAL);
+  if(vf->ready_state<STREAMFOUND)return(OV_EINVAL);
   if(!vf->seekable)
     return(OV_ENOSEEK); /* don't dump machine if we can't seek */
 
@@ -873,6 +933,7 @@
     int accblock=0;
     int thisblock;
     int eosflag;
+    int link;
 
     memset(&work_os,0,sizeof(work_os));/* so that it's safe to clear
 					  it later even if we don't
@@ -924,28 +985,23 @@
 	ogg_stream_clear(&work_os);
       }
 
-      if(vf->ready_state<STREAMSET){
-	int link;
-	
-	vf->current_serialno=ogg_page_serialno(&og);
-	for(link=0;link<vf->links;link++)
-	  if(vf->serialnos[link]==vf->current_serialno)break;
-	if(link==vf->links)goto seek_error; /* sign of a bogus stream.
-					       error out, leave
-					       machine uninitialized */
-	vf->current_link=link;
-	
-	ogg_stream_init(&vf->os,vf->current_serialno);
-	ogg_stream_reset(&vf->os); 
-	ogg_stream_init(&work_os,vf->current_serialno);
-	ogg_stream_reset(&work_os); 
-	vf->ready_state=STREAMSET;
-	
-      }
-    
-      ogg_stream_pagein(&vf->os,&og);
-      ogg_stream_pagein(&work_os,&og);
-      eosflag=ogg_page_eos(&og);
+      for(link=0;link<vf->links;link++)
+        if(vf->serialnos[link]==ogg_page_serialno(&og))break;
+      if((link<vf->links)){ /* Forget unknown packets..  */
+        if(vf->ready_state<STREAMSET){
+          vf->current_serialno=ogg_page_serialno(&og);
+          vf->current_link=link;
+          ogg_stream_init(&vf->os,vf->current_serialno);
+          ogg_stream_reset(&vf->os); 
+          ogg_stream_init(&work_os,vf->current_serialno);
+          ogg_stream_reset(&work_os); 
+          vf->ready_state=STREAMSET;
+        }
+
+        ogg_stream_pagein(&vf->os,&og);
+        ogg_stream_pagein(&work_os,&og);
+        eosflag=ogg_page_eos(&og);
+      }	
     }
   }
 

𸬵܆+޲mb+rz΂
k7&zzmb+ {.n++%lzwZ,j+n+zު笷rj)
'+a{
+v칻&ފ{ay,rجy'rzgǧ-b"Vފ	~)mz